//
//  CoreDataTableViewController.swift
//  TaskTime
//
//  Created by James Bamford on 20/04/2017.
//  Copyright © 2017 James Bamford. All rights reserved.
//

import UIKit
import CoreData

class CoreDataTableViewController<FetchRequestResultType:NSFetchRequestResult> : FetchedResultsTableViewController
{
  // Core Data Persistent Container - not sure we want this as a member
  //public var container: NSPersistentContainer? = (UIApplication.shared.delegate as? AppDelegate)?.persistentContainer
  
  // Fetched Results Controller that will be used as the model for this controller
  public var fetchedResultsController: NSFetchedResultsController<FetchRequestResultType>?
  {
    didSet
    {
      do
      {
        if let frc = fetchedResultsController
        {
          frc.delegate = self
          try frc.performFetch()
        }
        tableView.reloadData()
      }
      catch let error
      {
        dbgLog("NSFetchedResultsController.performFetch() failed: \(error)")
      }
    }
  }
  
  // MARK: UITableViewDataSource
  override func numberOfSections(in tableView: UITableView) -> Int
  {
    let numSections = fetchedResultsController?.sections?.count ?? 1
    dbgLog("frc=\(fetchedResultsController); numSections=\(numSections)")
    return numSections
  }
  
  override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
  {
    if let sections = fetchedResultsController?.sections, sections.count > 0
    {
      dbgLog("frc=\(fetchedResultsController); numRows=\(sections[section].numberOfObjects) in section=\(section)")
      return sections[section].numberOfObjects
    }
    else
    {
      dbgLog("failed: frc=\(fetchedResultsController); numRows=0 in section=\(section)")
      return 0
    }
  }
  
  override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String?
  {
    if let sections = fetchedResultsController?.sections, sections.count > 0
    {
      dbgLog("frc=\(fetchedResultsController); title=\(sections[section].name) in section=\(section)")
      return sections[section].name;
    }
    else
    {
      dbgLog("failed: frc=\(fetchedResultsController); title=nil in section=\(section)")
      return nil
    }
  }
  
  override func sectionIndexTitles(for tableView: UITableView) -> [String]?
  {
    return fetchedResultsController?.sectionIndexTitles
  }
  
  override func tableView(_ tableView: UITableView, sectionForSectionIndexTitle title: String, at index: Int) -> Int
  {
    let section = fetchedResultsController?.section(forSectionIndexTitle: title, at: index) ?? 0
    dbgLog("frc=\(fetchedResultsController); section=\(section) for title=\(title) at index=\(index)")
    return section
  }
}
