//
//  FetchedResultsTableViewController.swift
//
//  Created by Paul Hegarty on 2/3/2017.
//  Copyright © 2017 Stanford University. All rights reserved.
//

import UIKit
import CoreData

class FetchedResultsTableViewController: UITableViewController, NSFetchedResultsControllerDelegate
{
  public var tableViewRowAnimation : UITableViewRowAnimation = .fade
  
  // MARK: NSFetchedResultsControllerDelegate
  public func controllerWillChangeContent(_ controller: NSFetchedResultsController<NSFetchRequestResult>)
  {
    dbgLog("")
    debugOutputAllSectionAndRowCounts(controller, "controller=\(controller) before FetchedResults beginUpdates()");
    tableView.beginUpdates()
  }
  
  public func controller(
    _ controller: NSFetchedResultsController<NSFetchRequestResult>,
    didChange sectionInfo: NSFetchedResultsSectionInfo,
    atSectionIndex sectionIndex: Int,
    for type: NSFetchedResultsChangeType)
  {
    dbgLog("type=\(type); sectionIndex=\(sectionIndex)")
    switch type
    {
      case .insert:
        dbgLog("type is insert sectionIndex=\(sectionIndex)")
        tableView.insertSections([sectionIndex], with: tableViewRowAnimation)
      case .delete:
        dbgLog("type is delete sectionIndex=\(sectionIndex)")
        tableView.deleteSections([sectionIndex], with: tableViewRowAnimation)
      case .update:
        dbgLog("type is update.. currently unhandled")
        //tableView.reloadSections([sectionIndex], with: tableViewRowAnimation)
      default:  // nb. doesn't handle moving sections but iOS doesn't support that anyways for sections
        dbgLog("unhandled type=\(type)")
    }
  }
  
  func debugOutputAllSectionAndRowCounts(_ controller:NSFetchedResultsController<NSFetchRequestResult>, _ description:String)
  {
    // From TableView
    var numberOfSections = tableView.numberOfSections
    dbgLog("\(description) - FromTableView - numberOfSections=\(numberOfSections)")
    
    var sectionsSet = IndexSet.init(integersIn:0..<numberOfSections)
    for sectionIndex in sectionsSet
    {
      let numberOfRows = tableView.numberOfRows(inSection: sectionIndex)
      dbgLog("\(description) - section \(sectionIndex) has \(numberOfRows) rows")
    }
    
    // From FetchedResultsController
    numberOfSections = controller.sections!.count
    dbgLog("\(description) - FromFetchedResultsController: numberOfSections=\(numberOfSections)")
    
    sectionsSet = IndexSet.init(integersIn: 0..<numberOfSections)
    for sectionIndex in sectionsSet
    {
      let sectionInfo = controller.sections![sectionIndex]
      let sectionNameFromInfo = sectionInfo.name
      let numberOfRows = sectionInfo.numberOfObjects
      dbgLog("\(description) - FromFetchedResultsController: section=\(sectionIndex); has \(numberOfRows) rows (sectionNameFromInfo=\(sectionNameFromInfo)")
    }
  }
  
  public func controller(
    _ controller: NSFetchedResultsController<NSFetchRequestResult>,
    didChange anObject: Any,
    at indexPath: IndexPath?,
    for type: NSFetchedResultsChangeType,
    newIndexPath: IndexPath?)
  {
    dbgLog("type=\(type); indexPath=\(indexPath); newIndexPath=\(newIndexPath)")

    switch type
    {
      case .insert:
        dbgLog("type is insert.. newIndexPath=\(newIndexPath!) (section=\(newIndexPath!.section); row=\(newIndexPath!.row))")
        tableView.insertRows(at: [newIndexPath!], with: tableViewRowAnimation)
      case .delete:
        dbgLog("type is delete.. indexPath = \(indexPath!)")
        tableView.deleteRows(at: [indexPath!], with: tableViewRowAnimation)
      case .update:
        dbgLog("type is update.. indexPath = \(indexPath!)")
        tableView.reloadRows(at: [indexPath!], with: tableViewRowAnimation)
      case .move:
        dbgLog("type is move.. indexPath = \(indexPath!); newIndexPath=\(newIndexPath)")
        tableView.deleteRows(at: [indexPath!], with: tableViewRowAnimation)
        tableView.insertRows(at: [newIndexPath!], with: tableViewRowAnimation)
    }
  }
  
  public func controllerDidChangeContent(_ controller: NSFetchedResultsController<NSFetchRequestResult>)
  {
    dbgLog("controller=\(controller)")
    tableView.endUpdates()
    
    debugOutputAllSectionAndRowCounts(controller, "controller=\(controller) after FetchedResults endUpdates()");
  }
}
